"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


AssistantMessageRole = Literal["assistant"]
r"""The role of the messages author."""

AssistantMessageType = Literal["function"]
r"""The type of tool call."""


class AssistantMessageFunctionTypedDict(TypedDict):
    r"""The function specification"""

    name: str
    r"""The name of function"""
    arguments: str
    r"""The arguments of function in JSON schema format to call the function."""


class AssistantMessageFunction(BaseModel):
    r"""The function specification"""

    name: str
    r"""The name of function"""

    arguments: str
    r"""The arguments of function in JSON schema format to call the function."""


class ToolCallsTypedDict(TypedDict):
    id: str
    r"""The ID of tool call."""
    type: AssistantMessageType
    r"""The type of tool call."""
    function: AssistantMessageFunctionTypedDict
    r"""The function specification"""


class ToolCalls(BaseModel):
    id: str
    r"""The ID of tool call."""

    type: AssistantMessageType
    r"""The type of tool call."""

    function: AssistantMessageFunction
    r"""The function specification"""


class AssistantMessageTypedDict(TypedDict):
    role: AssistantMessageRole
    r"""The role of the messages author."""
    content: NotRequired[str]
    r"""The content of assistant message. Required unless `tool_calls` is specified."""
    name: NotRequired[str]
    r"""The name for the participant to distinguish between participants with the same role."""
    tool_calls: NotRequired[List[ToolCallsTypedDict]]


class AssistantMessage(BaseModel):
    role: AssistantMessageRole
    r"""The role of the messages author."""

    content: Optional[str] = None
    r"""The content of assistant message. Required unless `tool_calls` is specified."""

    name: Optional[str] = None
    r"""The name for the participant to distinguish between participants with the same role."""

    tool_calls: Optional[List[ToolCalls]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class TokenSequenceTypedDict(TypedDict):
    tokens: NotRequired[List[int]]
    r"""A List of token IDs."""


class TokenSequence(BaseModel):
    tokens: Optional[List[int]] = None
    r"""A List of token IDs."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .streamedcompletionschoice import (
    StreamedCompletionsChoice,
    StreamedCompletionsChoiceTypedDict,
)
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


StreamedCompletionsResultObject = Literal["text_completion"]
r"""The object type, which is always set to `text_completion`."""


class StreamedCompletionsResultUsageTypedDict(TypedDict):
    prompt_tokens: NotRequired[int]
    r"""Number of tokens in the prompt."""
    completion_tokens: NotRequired[int]
    r"""Number of tokens in the generated completions."""
    total_tokens: NotRequired[int]
    r"""Total number of tokens used in the request (`prompt_tokens` + `completion_tokens`)."""


class StreamedCompletionsResultUsage(BaseModel):
    prompt_tokens: Optional[int] = None
    r"""Number of tokens in the prompt."""

    completion_tokens: Optional[int] = None
    r"""Number of tokens in the generated completions."""

    total_tokens: Optional[int] = None
    r"""Total number of tokens used in the request (`prompt_tokens` + `completion_tokens`)."""


class StreamedCompletionsResultDataTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique ID of the completion."""
    object: NotRequired[StreamedCompletionsResultObject]
    r"""The object type, which is always set to `text_completion`."""
    choices: NotRequired[List[StreamedCompletionsChoiceTypedDict]]
    usage: NotRequired[Nullable[StreamedCompletionsResultUsageTypedDict]]
    created: NotRequired[int]
    r"""The Unix timestamp (in seconds) for when the token sampled."""


class StreamedCompletionsResultData(BaseModel):
    id: Optional[str] = None
    r"""A unique ID of the completion."""

    object: Optional[StreamedCompletionsResultObject] = None
    r"""The object type, which is always set to `text_completion`."""

    choices: Optional[List[StreamedCompletionsChoice]] = None

    usage: OptionalNullable[StreamedCompletionsResultUsage] = UNSET

    created: Optional[int] = None
    r"""The Unix timestamp (in seconds) for when the token sampled."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "object", "choices", "usage", "created"]
        nullable_fields = ["usage"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class StreamedCompletionsResultTypedDict(TypedDict):
    r"""Successfully generated completions."""

    data: NotRequired[StreamedCompletionsResultDataTypedDict]


class StreamedCompletionsResult(BaseModel):
    r"""Successfully generated completions."""

    data: Optional[StreamedCompletionsResultData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionschoice import CompletionsChoice, CompletionsChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


CompletionsResultObject = Literal["text_completion"]
r"""The object type, which is always set to `text_completion`."""


class CompletionsResultTypedDict(TypedDict):
    r"""Successfully generated completions."""

    id: NotRequired[str]
    r"""A unique ID of the completion."""
    object: NotRequired[CompletionsResultObject]
    r"""The object type, which is always set to `text_completion`."""
    choices: NotRequired[List[CompletionsChoiceTypedDict]]
    usage: NotRequired[UsageTypedDict]


class CompletionsResult(BaseModel):
    r"""Successfully generated completions."""

    id: Optional[str] = None
    r"""A unique ID of the completion."""

    object: Optional[CompletionsResultObject] = None
    r"""The object type, which is always set to `text_completion`."""

    choices: Optional[List[CompletionsChoice]] = None

    usage: Optional[Usage] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatchoice import ChatChoice, ChatChoiceTypedDict
from .usage import Usage, UsageTypedDict
from friendli.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


Object = Literal["chat.completion"]
r"""The object type, which is always set to `chat.completion`."""


class ChatResultTypedDict(TypedDict):
    r"""Successfully generated a chat response."""

    id: NotRequired[str]
    r"""A unique ID of the chat completion."""
    object: NotRequired[Object]
    r"""The object type, which is always set to `chat.completion`."""
    choices: NotRequired[List[ChatChoiceTypedDict]]
    usage: NotRequired[UsageTypedDict]
    created: NotRequired[int]
    r"""The Unix timestamp (in seconds) for when the generation completed."""


class ChatResult(BaseModel):
    r"""Successfully generated a chat response."""

    id: Optional[str] = None
    r"""A unique ID of the chat completion."""

    object: Optional[Object] = None
    r"""The object type, which is always set to `chat.completion`."""

    choices: Optional[List[ChatChoice]] = None

    usage: Optional[Usage] = None

    created: Optional[int] = None
    r"""The Unix timestamp (in seconds) for when the generation completed."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatlogprobs import ChatLogprobs, ChatLogprobsTypedDict
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


StreamedChatChoiceType = Literal["function"]
r"""The type of the tool."""


class StreamedChatChoiceFunctionTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the function to call."""
    arguments: NotRequired[str]
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class StreamedChatChoiceFunction(BaseModel):
    name: Optional[str] = None
    r"""The name of the function to call."""

    arguments: Optional[str] = None
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class StreamedChatChoiceToolCallsTypedDict(TypedDict):
    index: NotRequired[int]
    r"""The index of tool call being generated."""
    id: NotRequired[str]
    r"""The ID of the tool call."""
    type: NotRequired[StreamedChatChoiceType]
    r"""The type of the tool."""
    function: NotRequired[StreamedChatChoiceFunctionTypedDict]


class StreamedChatChoiceToolCalls(BaseModel):
    index: Optional[int] = None
    r"""The index of tool call being generated."""

    id: Optional[str] = None
    r"""The ID of the tool call."""

    type: Optional[StreamedChatChoiceType] = None
    r"""The type of the tool."""

    function: Optional[StreamedChatChoiceFunction] = None


class DeltaTypedDict(TypedDict):
    role: NotRequired[str]
    r"""Role of the generated message author, in this case `assistant`."""
    content: NotRequired[Nullable[str]]
    r"""The contents of the assistant message."""
    tool_calls: NotRequired[Nullable[StreamedChatChoiceToolCallsTypedDict]]


class Delta(BaseModel):
    role: Optional[str] = None
    r"""Role of the generated message author, in this case `assistant`."""

    content: OptionalNullable[str] = UNSET
    r"""The contents of the assistant message."""

    tool_calls: OptionalNullable[StreamedChatChoiceToolCalls] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["role", "content", "tool_calls"]
        nullable_fields = ["content", "tool_calls"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


StreamedChatChoiceFinishReason = Literal["stop", "length", "tool_calls", "null"]
r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
`length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
`tool_calls` means the API has generated tool calls.

"""


class StreamedChatChoiceTypedDict(TypedDict):
    index: NotRequired[int]
    r"""The index of the choice in the list of generated choices."""
    delta: NotRequired[DeltaTypedDict]
    finish_reason: NotRequired[Nullable[StreamedChatChoiceFinishReason]]
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """
    logprobs: NotRequired[Nullable[ChatLogprobsTypedDict]]
    r"""Log probability information for the choice."""


class StreamedChatChoice(BaseModel):
    index: Optional[int] = None
    r"""The index of the choice in the list of generated choices."""

    delta: Optional[Delta] = None

    finish_reason: OptionalNullable[StreamedChatChoiceFinishReason] = UNSET
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """

    logprobs: OptionalNullable[ChatLogprobs] = UNSET
    r"""Log probability information for the choice."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["index", "delta", "finish_reason", "logprobs"]
        nullable_fields = ["finish_reason", "logprobs"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


FileBuiltInToolType = Literal["file:text"]
r"""The type of the file parser tool. Only .txt and .pdf files are supported."""


class FileBuiltInToolTypedDict(TypedDict):
    type: FileBuiltInToolType
    r"""The type of the file parser tool. Only .txt and .pdf files are supported."""
    files: List[str]
    r"""A List of file IDs. For now, only one file is supported."""


class FileBuiltInTool(BaseModel):
    type: FileBuiltInToolType
    r"""The type of the file parser tool. Only .txt and .pdf files are supported."""

    files: List[str]
    r"""A List of file IDs. For now, only one file is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ServerlessKnowledgeRetrieveBodyTypedDict(TypedDict):
    query: NotRequired[str]
    r"""A text string used to find relevant information within the knowledge-base."""
    k: NotRequired[int]
    r"""Maximum number of top-ranked knowledge-base entries to return in results."""
    knowledge_ids: NotRequired[List[str]]
    r"""A List of knowledge-base IDs. For now, only one knowledge-base is supported."""


class ServerlessKnowledgeRetrieveBody(BaseModel):
    query: Optional[str] = None
    r"""A text string used to find relevant information within the knowledge-base."""

    k: Optional[int] = None
    r"""Maximum number of top-ranked knowledge-base entries to return in results."""

    knowledge_ids: Annotated[
        Optional[List[str]], pydantic.Field(alias="knowledgeIds")
    ] = None
    r"""A List of knowledge-base IDs. For now, only one knowledge-base is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ResultsTypedDict(TypedDict):
    text: NotRequired[str]
    r"""Retrieved text matching the search query."""
    score: NotRequired[float]
    r"""Numerical relevance score of the retrieved result."""
    content_chunk_id: NotRequired[str]
    r"""ID of specific text segment."""
    content_id: NotRequired[str]
    r"""ID of content in knowledge-base."""


class Results(BaseModel):
    text: Optional[str] = None
    r"""Retrieved text matching the search query."""

    score: Optional[float] = None
    r"""Numerical relevance score of the retrieved result."""

    content_chunk_id: Optional[str] = None
    r"""ID of specific text segment."""

    content_id: Optional[str] = None
    r"""ID of content in knowledge-base."""


class KnowledgeRetrieveResultTypedDict(TypedDict):
    r"""Successfully retrieved contexts from vector store."""

    results: NotRequired[List[ResultsTypedDict]]
    r"""A list of result."""


class KnowledgeRetrieveResult(BaseModel):
    r"""Successfully retrieved contexts from vector store."""

    results: Optional[List[Results]] = None
    r"""A list of result."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class TopLogprobsTypedDict(TypedDict):
    pass


class TopLogprobs(BaseModel):
    pass


class CompletionsLogprobsTypedDict(TypedDict):
    r"""Log probability information for the choice."""

    text_offset: NotRequired[List[int]]
    r"""The starting character position of each token in the generated text, useful for mapping tokens back to their exact location for detailed analysis."""
    token_logprobs: NotRequired[List[float]]
    r"""The log probabilities of each generated token, indicating the model's confidence in selecting each token."""
    tokens: NotRequired[List[str]]
    r"""A list of individual tokens generated in the completion, representing segments of text such as words or pieces of words."""
    top_logprobs: NotRequired[List[TopLogprobsTypedDict]]
    r"""A list of dictionaries, where each dictionary represents the top alternative tokens considered by the model at a specific position in the generated text, along with their log probabilities. The number of items in each dictionary matches the value of `logprobs`."""


class CompletionsLogprobs(BaseModel):
    r"""Log probability information for the choice."""

    text_offset: Optional[List[int]] = None
    r"""The starting character position of each token in the generated text, useful for mapping tokens back to their exact location for detailed analysis."""

    token_logprobs: Optional[List[float]] = None
    r"""The log probabilities of each generated token, indicating the model's confidence in selecting each token."""

    tokens: Optional[List[str]] = None
    r"""A list of individual tokens generated in the completion, representing segments of text such as words or pieces of words."""

    top_logprobs: Optional[List[TopLogprobs]] = None
    r"""A list of dictionaries, where each dictionary represents the top alternative tokens considered by the model at a specific position in the generated text, along with their log probabilities. The number of items in each dictionary matches the value of `logprobs`."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ChatLogprobsTopLogprobsTypedDict(TypedDict):
    token: NotRequired[str]
    r"""The token."""
    logprob: NotRequired[float]
    r"""The log probability of this token."""
    bytes: NotRequired[Nullable[List[int]]]
    r"""A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."""


class ChatLogprobsTopLogprobs(BaseModel):
    token: Optional[str] = None
    r"""The token."""

    logprob: Optional[float] = None
    r"""The log probability of this token."""

    bytes: OptionalNullable[List[int]] = UNSET
    r"""A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["token", "logprob", "bytes"]
        nullable_fields = ["bytes"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ContentTypedDict(TypedDict):
    token: NotRequired[str]
    r"""The token."""
    logprob: NotRequired[float]
    r"""The log probability of this token."""
    bytes: NotRequired[Nullable[List[int]]]
    r"""A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."""
    top_logprobs: NotRequired[List[ChatLogprobsTopLogprobsTypedDict]]
    r"""List of the most likely tokens and their log probability, at this token position."""


class Content(BaseModel):
    token: Optional[str] = None
    r"""The token."""

    logprob: Optional[float] = None
    r"""The log probability of this token."""

    bytes: OptionalNullable[List[int]] = UNSET
    r"""A list of integers representing the UTF-8 bytes representation of the token. Useful in instances where characters are represented by multiple tokens and their byte representations must be combined to generate the correct text representation. Can be `null` if there is no bytes representation for the token."""

    top_logprobs: Optional[List[ChatLogprobsTopLogprobs]] = None
    r"""List of the most likely tokens and their log probability, at this token position."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["token", "logprob", "bytes", "top_logprobs"]
        nullable_fields = ["bytes"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ChatLogprobsTypedDict(TypedDict):
    r"""Log probability information for the choice."""

    content: NotRequired[Nullable[List[ContentTypedDict]]]
    r"""A list of message content tokens with log probability information."""


class ChatLogprobs(BaseModel):
    r"""Log probability information for the choice."""

    content: OptionalNullable[List[Content]] = UNSET
    r"""A list of message content tokens with log probability information."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["content"]
        nullable_fields = ["content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from friendli.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


Name = Literal[
    "math:calculator",
    "math:statistics",
    "math:calendar",
    "web:search",
    "web:url",
    "code:python-interpreter",
    "file:text",
]
r"""The name of the built-in tool."""

Status = Literal["STARTED", "UPDATING", "ENDED", "ERRORED"]
r"""Indicates the current execution status of the tool."""


class StreamedToolAssistedChatToolStatusParametersTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the tool’s function parameter."""
    value: NotRequired[str]
    r"""The value of the tool’s function parameter."""


class StreamedToolAssistedChatToolStatusParameters(BaseModel):
    name: Optional[str] = None
    r"""The name of the tool’s function parameter."""

    value: Optional[str] = None
    r"""The value of the tool’s function parameter."""


class FilesTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the file generated by the tool’s execution."""
    url: NotRequired[str]
    r"""URL of the file generated by the tool’s execution."""


class Files(BaseModel):
    name: Optional[str] = None
    r"""The name of the file generated by the tool’s execution."""

    url: Optional[str] = None
    r"""URL of the file generated by the tool’s execution."""


class ErrorTypedDict(TypedDict):
    type: NotRequired[str]
    r"""The type of error encountered during the tool’s execution."""
    msg: NotRequired[str]
    r"""The message of error."""


class Error(BaseModel):
    type: Optional[str] = None
    r"""The type of error encountered during the tool’s execution."""

    msg: Optional[str] = None
    r"""The message of error."""


class StreamedToolAssistedChatToolStatusDataTypedDict(TypedDict):
    tool_call_id: NotRequired[str]
    r"""The ID of the tool call."""
    name: NotRequired[Name]
    r"""The name of the built-in tool."""
    status: NotRequired[Status]
    r"""Indicates the current execution status of the tool."""
    parameters: NotRequired[List[StreamedToolAssistedChatToolStatusParametersTypedDict]]
    result: NotRequired[Nullable[str]]
    r"""The output from the tool’s execution."""
    files: NotRequired[Nullable[List[FilesTypedDict]]]
    message: NotRequired[Nullable[str]]
    r"""Message generated by the tool’s execution."""
    error: NotRequired[Nullable[ErrorTypedDict]]
    timestamp: NotRequired[float]
    r"""The Unix timestamp (in seconds) for when the event occurred."""


class StreamedToolAssistedChatToolStatusData(BaseModel):
    tool_call_id: Optional[str] = None
    r"""The ID of the tool call."""

    name: Optional[Name] = None
    r"""The name of the built-in tool."""

    status: Optional[Status] = None
    r"""Indicates the current execution status of the tool."""

    parameters: Optional[List[StreamedToolAssistedChatToolStatusParameters]] = None

    result: OptionalNullable[str] = UNSET
    r"""The output from the tool’s execution."""

    files: OptionalNullable[List[Files]] = UNSET

    message: OptionalNullable[str] = UNSET
    r"""Message generated by the tool’s execution."""

    error: OptionalNullable[Error] = UNSET

    timestamp: Optional[float] = None
    r"""The Unix timestamp (in seconds) for when the event occurred."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "tool_call_id",
            "name",
            "status",
            "parameters",
            "result",
            "files",
            "message",
            "error",
            "timestamp",
        ]
        nullable_fields = ["result", "files", "message", "error"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class StreamedToolAssistedChatToolStatusTypedDict(TypedDict):
    event: Literal["tool_status"]
    data: NotRequired[StreamedToolAssistedChatToolStatusDataTypedDict]


class StreamedToolAssistedChatToolStatus(BaseModel):
    EVENT: Annotated[
        Annotated[
            Optional[Literal["tool_status"]],
            AfterValidator(validate_const("tool_status")),
        ],
        pydantic.Field(alias="event"),
    ] = "tool_status"

    data: Optional[StreamedToolAssistedChatToolStatusData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from friendli.chat import Chat
from friendli.completions import Completions
from friendli.rag import Rag
from friendli.token import Token
from friendli.toolassistedchat import ToolAssistedChat


class Serverless(BaseSDK):
    chat: Chat
    tool_assisted_chat: ToolAssistedChat
    completions: Completions
    token: Token
    rag: Rag

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.chat = Chat(self.sdk_configuration)
        self.tool_assisted_chat = ToolAssistedChat(self.sdk_configuration)
        self.completions = Completions(self.sdk_configuration)
        self.token = Token(self.sdk_configuration)
        self.rag = Rag(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
import pydantic
from typing import Literal
from typing_extensions import Annotated, TypedDict


Type = Literal["json_schema"]
r"""The type of the response format: `json_schema`"""


class SchemaTypedDict(TypedDict):
    r"""The schema for the response format, described as a JSON Schema object."""


class Schema(BaseModel):
    r"""The schema for the response format, described as a JSON Schema object."""


class JSONSchemaTypedDict(TypedDict):
    schema_: SchemaTypedDict
    r"""The schema for the response format, described as a JSON Schema object."""


class JSONSchema(BaseModel):
    schema_: Annotated[Schema, pydantic.Field(alias="schema")]
    r"""The schema for the response format, described as a JSON Schema object."""


class ResponseFormatJSONSchemaTypedDict(TypedDict):
    type: Type
    r"""The type of the response format: `json_schema`"""
    json_schema: JSONSchemaTypedDict


class ResponseFormatJSONSchema(BaseModel):
    type: Type
    r"""The type of the response format: `json_schema`"""

    json_schema: JSONSchema



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DedicatedDetokenizationBodyTypedDict(TypedDict):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""
    tokens: NotRequired[List[int]]
    r"""A token sequence to detokenize."""


class DedicatedDetokenizationBody(BaseModel):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format."""

    tokens: Optional[List[int]] = None
    r"""A token sequence to detokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


ResponseFormatJSONObjectType = Literal["json_object"]
r"""The type of the response format: `json_object`"""


class ResponseFormatJSONObjectTypedDict(TypedDict):
    type: ResponseFormatJSONObjectType
    r"""The type of the response format: `json_object`"""
    schema_: NotRequired[Nullable[str]]
    r"""The serialized JSON schema string to enforce as the response format.
    Note that recursive definitions are not supported. Optional properties are also unsupported; all properties defined under `{ \"type\": \"object\" }` will be generated, regardless of their inclusion in the required field.

    """


class ResponseFormatJSONObject(BaseModel):
    type: ResponseFormatJSONObjectType
    r"""The type of the response format: `json_object`"""

    schema_: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="schema",
        ),
    ] = UNSET
    r"""The serialized JSON schema string to enforce as the response format.
    Note that recursive definitions are not supported. Optional properties are also unsupported; all properties defined under `{ \"type\": \"object\" }` will be generated, regardless of their inclusion in the required field.

    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["schema"]
        nullable_fields = ["schema"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .assistantmessage import (
    AssistantMessage,
    AssistantMessageFunction,
    AssistantMessageFunctionTypedDict,
    AssistantMessageRole,
    AssistantMessageType,
    AssistantMessageTypedDict,
    ToolCalls,
    ToolCallsTypedDict,
)
from .chatchoice import (
    ChatChoice,
    ChatChoiceFunction,
    ChatChoiceFunctionTypedDict,
    ChatChoiceMessage,
    ChatChoiceMessageTypedDict,
    ChatChoiceToolCalls,
    ChatChoiceToolCallsTypedDict,
    ChatChoiceType,
    ChatChoiceTypedDict,
    FinishReason,
)
from .chatlogprobs import (
    ChatLogprobs,
    ChatLogprobsTopLogprobs,
    ChatLogprobsTopLogprobsTypedDict,
    ChatLogprobsTypedDict,
    Content,
    ContentTypedDict,
)
from .chatresult import ChatResult, ChatResultTypedDict, Object
from .completionsbodywithprompt import (
    CompletionsBodyWithPrompt,
    CompletionsBodyWithPromptStreamOptions,
    CompletionsBodyWithPromptStreamOptionsTypedDict,
    CompletionsBodyWithPromptTypedDict,
    Prompt,
    PromptTypedDict,
)
from .completionsbodywithtokens import (
    CompletionsBodyWithTokens,
    CompletionsBodyWithTokensStreamOptions,
    CompletionsBodyWithTokensStreamOptionsTypedDict,
    CompletionsBodyWithTokensTypedDict,
)
from .completionschoice import (
    CompletionsChoice,
    CompletionsChoiceFinishReason,
    CompletionsChoiceTypedDict,
)
from .completionslogprobs import (
    CompletionsLogprobs,
    CompletionsLogprobsTypedDict,
    TopLogprobs,
    TopLogprobsTypedDict,
)
from .completionsresult import (
    CompletionsResult,
    CompletionsResultObject,
    CompletionsResultTypedDict,
)
from .dedicatedchatcompletebody import (
    DedicatedChatCompleteBody,
    DedicatedChatCompleteBodyLogitBias,
    DedicatedChatCompleteBodyLogitBiasTypedDict,
    DedicatedChatCompleteBodyStreamOptions,
    DedicatedChatCompleteBodyStreamOptionsTypedDict,
    DedicatedChatCompleteBodyToolChoice,
    DedicatedChatCompleteBodyToolChoiceFunction,
    DedicatedChatCompleteBodyToolChoiceFunctionTypedDict,
    DedicatedChatCompleteBodyToolChoiceObject,
    DedicatedChatCompleteBodyToolChoiceObjectTypedDict,
    DedicatedChatCompleteBodyToolChoiceType,
    DedicatedChatCompleteBodyToolChoiceTypedDict,
    DedicatedChatCompleteBodyTypedDict,
)
from .dedicatedchatcompleteop import (
    DedicatedChatCompleteRequest,
    DedicatedChatCompleteRequestTypedDict,
)
from .dedicatedchatstreambody import (
    DedicatedChatStreamBody,
    DedicatedChatStreamBodyLogitBias,
    DedicatedChatStreamBodyLogitBiasTypedDict,
    DedicatedChatStreamBodyStreamOptions,
    DedicatedChatStreamBodyStreamOptionsTypedDict,
    DedicatedChatStreamBodyToolChoice,
    DedicatedChatStreamBodyToolChoiceFunction,
    DedicatedChatStreamBodyToolChoiceFunctionTypedDict,
    DedicatedChatStreamBodyToolChoiceObject,
    DedicatedChatStreamBodyToolChoiceObjectTypedDict,
    DedicatedChatStreamBodyToolChoiceType,
    DedicatedChatStreamBodyToolChoiceTypedDict,
    DedicatedChatStreamBodyTypedDict,
)
from .dedicatedchatstreamop import (
    DedicatedChatStreamRequest,
    DedicatedChatStreamRequestTypedDict,
)
from .dedicatedcompletionscompletebody import (
    CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyPrompt,
    CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyPromptTypedDict,
    CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyStreamOptions,
    CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyStreamOptionsTypedDict,
    CompletionsBodyWithTokensDedicatedCompletionsCompleteBodyStreamOptions,
    CompletionsBodyWithTokensDedicatedCompletionsCompleteBodyStreamOptionsTypedDict,
    DedicatedCompletionsCompleteBody,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithPrompt,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithPromptTypedDict,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithTokens,
    DedicatedCompletionsCompleteBodyCompletionsBodyWithTokensTypedDict,
    DedicatedCompletionsCompleteBodyTypedDict,
)
from .dedicatedcompletionscompleteop import (
    DedicatedCompletionsCompleteRequest,
    DedicatedCompletionsCompleteRequestTypedDict,
)
from .dedicatedcompletionsstreambody import (
    CompletionsBodyWithPromptDedicatedCompletionsStreamBodyPrompt,
    CompletionsBodyWithPromptDedicatedCompletionsStreamBodyPromptTypedDict,
    CompletionsBodyWithPromptDedicatedCompletionsStreamBodyStreamOptions,
    CompletionsBodyWithPromptDedicatedCompletionsStreamBodyStreamOptionsTypedDict,
    CompletionsBodyWithTokensDedicatedCompletionsStreamBodyStreamOptions,
    CompletionsBodyWithTokensDedicatedCompletionsStreamBodyStreamOptionsTypedDict,
    DedicatedCompletionsStreamBody,
    DedicatedCompletionsStreamBodyCompletionsBodyWithPrompt,
    DedicatedCompletionsStreamBodyCompletionsBodyWithPromptTypedDict,
    DedicatedCompletionsStreamBodyCompletionsBodyWithTokens,
    DedicatedCompletionsStreamBodyCompletionsBodyWithTokensTypedDict,
    DedicatedCompletionsStreamBodyTypedDict,
)
from .dedicatedcompletionsstreamop import (
    DedicatedCompletionsStreamRequest,
    DedicatedCompletionsStreamRequestTypedDict,
)
from .dedicateddetokenizationbody import (
    DedicatedDetokenizationBody,
    DedicatedDetokenizationBodyTypedDict,
)
from .dedicateddetokenizationop import (
    DedicatedDetokenizationRequest,
    DedicatedDetokenizationRequestTypedDict,
)
from .dedicatedtokenizationbody import (
    DedicatedTokenizationBody,
    DedicatedTokenizationBodyTypedDict,
)
from .dedicatedtokenizationop import (
    DedicatedTokenizationRequest,
    DedicatedTokenizationRequestTypedDict,
)
from .detokenizationresult import DetokenizationResult, DetokenizationResultTypedDict
from .filebuiltintool import (
    FileBuiltInTool,
    FileBuiltInToolType,
    FileBuiltInToolTypedDict,
)
from .function import Function, FunctionTypedDict, Parameters, ParametersTypedDict
from .functiontool import FunctionTool, FunctionToolType, FunctionToolTypedDict
from .knowledgeretrieveresult import (
    KnowledgeRetrieveResult,
    KnowledgeRetrieveResultTypedDict,
    Results,
    ResultsTypedDict,
)
from .message import Message, MessageTypedDict
from .otherbuiltintool import (
    OtherBuiltInTool,
    OtherBuiltInToolType,
    OtherBuiltInToolTypedDict,
)
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from .responseformatjsonobject import (
    ResponseFormatJSONObject,
    ResponseFormatJSONObjectType,
    ResponseFormatJSONObjectTypedDict,
)
from .responseformatjsonschema import (
    JSONSchema,
    JSONSchemaTypedDict,
    ResponseFormatJSONSchema,
    ResponseFormatJSONSchemaTypedDict,
    Schema,
    SchemaTypedDict,
    Type,
)
from .responseformatregex import (
    ResponseFormatRegex,
    ResponseFormatRegexType,
    ResponseFormatRegexTypedDict,
)
from .responseformattext import (
    ResponseFormatText,
    ResponseFormatTextType,
    ResponseFormatTextTypedDict,
)
from .sdkerror import SDKError
from .security import Security, SecurityTypedDict
from .serverlesschatcompletebody import (
    LogitBias,
    LogitBiasTypedDict,
    ServerlessChatCompleteBody,
    ServerlessChatCompleteBodyTypedDict,
    StreamOptions,
    StreamOptionsTypedDict,
    ToolChoice,
    ToolChoiceFunction,
    ToolChoiceFunctionTypedDict,
    ToolChoiceObject,
    ToolChoiceObjectTypedDict,
    ToolChoiceType,
    ToolChoiceTypedDict,
)
from .serverlesschatcompleteop import (
    ServerlessChatCompleteRequest,
    ServerlessChatCompleteRequestTypedDict,
)
from .serverlesschatstreambody import (
    ServerlessChatStreamBody,
    ServerlessChatStreamBodyLogitBias,
    ServerlessChatStreamBodyLogitBiasTypedDict,
    ServerlessChatStreamBodyStreamOptions,
    ServerlessChatStreamBodyStreamOptionsTypedDict,
    ServerlessChatStreamBodyToolChoice,
    ServerlessChatStreamBodyToolChoiceFunction,
    ServerlessChatStreamBodyToolChoiceFunctionTypedDict,
    ServerlessChatStreamBodyToolChoiceObject,
    ServerlessChatStreamBodyToolChoiceObjectTypedDict,
    ServerlessChatStreamBodyToolChoiceType,
    ServerlessChatStreamBodyToolChoiceTypedDict,
    ServerlessChatStreamBodyTypedDict,
)
from .serverlesschatstreamop import (
    ServerlessChatStreamRequest,
    ServerlessChatStreamRequestTypedDict,
)
from .serverlesscompletionscompletebody import (
    ServerlessCompletionsCompleteBody,
    ServerlessCompletionsCompleteBodyTypedDict,
)
from .serverlesscompletionscompleteop import (
    ServerlessCompletionsCompleteRequest,
    ServerlessCompletionsCompleteRequestTypedDict,
)
from .serverlesscompletionsstreambody import (
    CompletionsBodyWithPromptPrompt,
    CompletionsBodyWithPromptPromptTypedDict,
    CompletionsBodyWithPromptServerlessCompletionsStreamBodyStreamOptions,
    CompletionsBodyWithPromptServerlessCompletionsStreamBodyStreamOptionsTypedDict,
    CompletionsBodyWithTokensServerlessCompletionsStreamBodyStreamOptions,
    CompletionsBodyWithTokensServerlessCompletionsStreamBodyStreamOptionsTypedDict,
    ServerlessCompletionsStreamBody,
    ServerlessCompletionsStreamBodyCompletionsBodyWithPrompt,
    ServerlessCompletionsStreamBodyCompletionsBodyWithPromptTypedDict,
    ServerlessCompletionsStreamBodyCompletionsBodyWithTokens,
    ServerlessCompletionsStreamBodyCompletionsBodyWithTokensTypedDict,
    ServerlessCompletionsStreamBodyTypedDict,
)
from .serverlesscompletionsstreamop import (
    ServerlessCompletionsStreamRequest,
    ServerlessCompletionsStreamRequestTypedDict,
)
from .serverlessdetokenizationbody import (
    ServerlessDetokenizationBody,
    ServerlessDetokenizationBodyTypedDict,
)
from .serverlessdetokenizationop import (
    ServerlessDetokenizationRequest,
    ServerlessDetokenizationRequestTypedDict,
)
from .serverlessknowledgeretrievebody import (
    ServerlessKnowledgeRetrieveBody,
    ServerlessKnowledgeRetrieveBodyTypedDict,
)
from .serverlessknowledgeretrieveop import (
    ServerlessKnowledgeRetrieveRequest,
    ServerlessKnowledgeRetrieveRequestTypedDict,
)
from .serverlesstokenizationbody import (
    ServerlessTokenizationBody,
    ServerlessTokenizationBodyTypedDict,
)
from .serverlesstokenizationop import (
    ServerlessTokenizationRequest,
    ServerlessTokenizationRequestTypedDict,
)
from .serverlesstoolassistedchatcompletebody import (
    ServerlessToolAssistedChatCompleteBody,
    ServerlessToolAssistedChatCompleteBodyLogitBias,
    ServerlessToolAssistedChatCompleteBodyLogitBiasTypedDict,
    ServerlessToolAssistedChatCompleteBodyStreamOptions,
    ServerlessToolAssistedChatCompleteBodyStreamOptionsTypedDict,
    ServerlessToolAssistedChatCompleteBodyToolChoice,
    ServerlessToolAssistedChatCompleteBodyToolChoiceFunction,
    ServerlessToolAssistedChatCompleteBodyToolChoiceFunctionTypedDict,
    ServerlessToolAssistedChatCompleteBodyToolChoiceObject,
    ServerlessToolAssistedChatCompleteBodyToolChoiceObjectTypedDict,
    ServerlessToolAssistedChatCompleteBodyToolChoiceType,
    ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatCompleteBodyTypedDict,
)
from .serverlesstoolassistedchatcompleteop import (
    ServerlessToolAssistedChatCompleteRequest,
    ServerlessToolAssistedChatCompleteRequestTypedDict,
)
from .serverlesstoolassistedchatstreambody import (
    ServerlessToolAssistedChatStreamBody,
    ServerlessToolAssistedChatStreamBodyLogitBias,
    ServerlessToolAssistedChatStreamBodyLogitBiasTypedDict,
    ServerlessToolAssistedChatStreamBodyStreamOptions,
    ServerlessToolAssistedChatStreamBodyStreamOptionsTypedDict,
    ServerlessToolAssistedChatStreamBodyToolChoice,
    ServerlessToolAssistedChatStreamBodyToolChoiceFunction,
    ServerlessToolAssistedChatStreamBodyToolChoiceFunctionTypedDict,
    ServerlessToolAssistedChatStreamBodyToolChoiceObject,
    ServerlessToolAssistedChatStreamBodyToolChoiceObjectTypedDict,
    ServerlessToolAssistedChatStreamBodyToolChoiceType,
    ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatStreamBodyTypedDict,
)
from .serverlesstoolassistedchatstreamop import (
    ServerlessToolAssistedChatStreamRequest,
    ServerlessToolAssistedChatStreamRequestTypedDict,
)
from .streamedchatchoice import (
    Delta,
    DeltaTypedDict,
    StreamedChatChoice,
    StreamedChatChoiceFinishReason,
    StreamedChatChoiceFunction,
    StreamedChatChoiceFunctionTypedDict,
    StreamedChatChoiceToolCalls,
    StreamedChatChoiceToolCallsTypedDict,
    StreamedChatChoiceType,
    StreamedChatChoiceTypedDict,
)
from .streamedchatresult import (
    Data,
    DataTypedDict,
    StreamedChatResult,
    StreamedChatResultObject,
    StreamedChatResultTypedDict,
    StreamedChatResultUsage,
    StreamedChatResultUsageTypedDict,
)
from .streamedcompletionschoice import (
    StreamedCompletionsChoice,
    StreamedCompletionsChoiceFinishReason,
    StreamedCompletionsChoiceTypedDict,
)
from .streamedcompletionsresult import (
    StreamedCompletionsResult,
    StreamedCompletionsResultData,
    StreamedCompletionsResultDataTypedDict,
    StreamedCompletionsResultObject,
    StreamedCompletionsResultTypedDict,
    StreamedCompletionsResultUsage,
    StreamedCompletionsResultUsageTypedDict,
)
from .streamedtoolassistedchatresult import (
    StreamedToolAssistedChatResult,
    StreamedToolAssistedChatResultTypedDict,
)
from .streamedtoolassistedchattoken import (
    StreamedToolAssistedChatToken,
    StreamedToolAssistedChatTokenData,
    StreamedToolAssistedChatTokenDataTypedDict,
    StreamedToolAssistedChatTokenObject,
    StreamedToolAssistedChatTokenTypedDict,
    StreamedToolAssistedChatTokenUsage,
    StreamedToolAssistedChatTokenUsageTypedDict,
)
from .streamedtoolassistedchattoolstatus import (
    Error,
    ErrorTypedDict,
    Files,
    FilesTypedDict,
    Name,
    Status,
    StreamedToolAssistedChatToolStatus,
    StreamedToolAssistedChatToolStatusData,
    StreamedToolAssistedChatToolStatusDataTypedDict,
    StreamedToolAssistedChatToolStatusParameters,
    StreamedToolAssistedChatToolStatusParametersTypedDict,
    StreamedToolAssistedChatToolStatusTypedDict,
)
from .systemmessage import Role, SystemMessage, SystemMessageTypedDict
from .tokenizationresult import TokenizationResult, TokenizationResultTypedDict
from .tokensequence import TokenSequence, TokenSequenceTypedDict
from .tool import Tool, ToolType, ToolTypedDict
from .toolassistedchattool import ToolAssistedChatTool, ToolAssistedChatToolTypedDict
from .toolmessage import ToolMessage, ToolMessageRole, ToolMessageTypedDict
from .usage import Usage, UsageTypedDict
from .usermessage import UserMessage, UserMessageRole, UserMessageTypedDict

__all__ = [
    "AssistantMessage",
    "AssistantMessageFunction",
    "AssistantMessageFunctionTypedDict",
    "AssistantMessageRole",
    "AssistantMessageType",
    "AssistantMessageTypedDict",
    "ChatChoice",
    "ChatChoiceFunction",
    "ChatChoiceFunctionTypedDict",
    "ChatChoiceMessage",
    "ChatChoiceMessageTypedDict",
    "ChatChoiceToolCalls",
    "ChatChoiceToolCallsTypedDict",
    "ChatChoiceType",
    "ChatChoiceTypedDict",
    "ChatLogprobs",
    "ChatLogprobsTopLogprobs",
    "ChatLogprobsTopLogprobsTypedDict",
    "ChatLogprobsTypedDict",
    "ChatResult",
    "ChatResultTypedDict",
    "CompletionsBodyWithPrompt",
    "CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyPrompt",
    "CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyPromptTypedDict",
    "CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyStreamOptions",
    "CompletionsBodyWithPromptDedicatedCompletionsCompleteBodyStreamOptionsTypedDict",
    "CompletionsBodyWithPromptDedicatedCompletionsStreamBodyPrompt",
    "CompletionsBodyWithPromptDedicatedCompletionsStreamBodyPromptTypedDict",
    "CompletionsBodyWithPromptDedicatedCompletionsStreamBodyStreamOptions",
    "CompletionsBodyWithPromptDedicatedCompletionsStreamBodyStreamOptionsTypedDict",
    "CompletionsBodyWithPromptPrompt",
    "CompletionsBodyWithPromptPromptTypedDict",
    "CompletionsBodyWithPromptServerlessCompletionsStreamBodyStreamOptions",
    "CompletionsBodyWithPromptServerlessCompletionsStreamBodyStreamOptionsTypedDict",
    "CompletionsBodyWithPromptStreamOptions",
    "CompletionsBodyWithPromptStreamOptionsTypedDict",
    "CompletionsBodyWithPromptTypedDict",
    "CompletionsBodyWithTokens",
    "CompletionsBodyWithTokensDedicatedCompletionsCompleteBodyStreamOptions",
    "CompletionsBodyWithTokensDedicatedCompletionsCompleteBodyStreamOptionsTypedDict",
    "CompletionsBodyWithTokensDedicatedCompletionsStreamBodyStreamOptions",
    "CompletionsBodyWithTokensDedicatedCompletionsStreamBodyStreamOptionsTypedDict",
    "CompletionsBodyWithTokensServerlessCompletionsStreamBodyStreamOptions",
    "CompletionsBodyWithTokensServerlessCompletionsStreamBodyStreamOptionsTypedDict",
    "CompletionsBodyWithTokensStreamOptions",
    "CompletionsBodyWithTokensStreamOptionsTypedDict",
    "CompletionsBodyWithTokensTypedDict",
    "CompletionsChoice",
    "CompletionsChoiceFinishReason",
    "CompletionsChoiceTypedDict",
    "CompletionsLogprobs",
    "CompletionsLogprobsTypedDict",
    "CompletionsResult",
    "CompletionsResultObject",
    "CompletionsResultTypedDict",
    "Content",
    "ContentTypedDict",
    "Data",
    "DataTypedDict",
    "DedicatedChatCompleteBody",
    "DedicatedChatCompleteBodyLogitBias",
    "DedicatedChatCompleteBodyLogitBiasTypedDict",
    "DedicatedChatCompleteBodyStreamOptions",
    "DedicatedChatCompleteBodyStreamOptionsTypedDict",
    "DedicatedChatCompleteBodyToolChoice",
    "DedicatedChatCompleteBodyToolChoiceFunction",
    "DedicatedChatCompleteBodyToolChoiceFunctionTypedDict",
    "DedicatedChatCompleteBodyToolChoiceObject",
    "DedicatedChatCompleteBodyToolChoiceObjectTypedDict",
    "DedicatedChatCompleteBodyToolChoiceType",
    "DedicatedChatCompleteBodyToolChoiceTypedDict",
    "DedicatedChatCompleteBodyTypedDict",
    "DedicatedChatCompleteRequest",
    "DedicatedChatCompleteRequestTypedDict",
    "DedicatedChatStreamBody",
    "DedicatedChatStreamBodyLogitBias",
    "DedicatedChatStreamBodyLogitBiasTypedDict",
    "DedicatedChatStreamBodyStreamOptions",
    "DedicatedChatStreamBodyStreamOptionsTypedDict",
    "DedicatedChatStreamBodyToolChoice",
    "DedicatedChatStreamBodyToolChoiceFunction",
    "DedicatedChatStreamBodyToolChoiceFunctionTypedDict",
    "DedicatedChatStreamBodyToolChoiceObject",
    "DedicatedChatStreamBodyToolChoiceObjectTypedDict",
    "DedicatedChatStreamBodyToolChoiceType",
    "DedicatedChatStreamBodyToolChoiceTypedDict",
    "DedicatedChatStreamBodyTypedDict",
    "DedicatedChatStreamRequest",
    "DedicatedChatStreamRequestTypedDict",
    "DedicatedCompletionsCompleteBody",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithPrompt",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithPromptTypedDict",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithTokens",
    "DedicatedCompletionsCompleteBodyCompletionsBodyWithTokensTypedDict",
    "DedicatedCompletionsCompleteBodyTypedDict",
    "DedicatedCompletionsCompleteRequest",
    "DedicatedCompletionsCompleteRequestTypedDict",
    "DedicatedCompletionsStreamBody",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithPrompt",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithPromptTypedDict",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithTokens",
    "DedicatedCompletionsStreamBodyCompletionsBodyWithTokensTypedDict",
    "DedicatedCompletionsStreamBodyTypedDict",
    "DedicatedCompletionsStreamRequest",
    "DedicatedCompletionsStreamRequestTypedDict",
    "DedicatedDetokenizationBody",
    "DedicatedDetokenizationBodyTypedDict",
    "DedicatedDetokenizationRequest",
    "DedicatedDetokenizationRequestTypedDict",
    "DedicatedTokenizationBody",
    "DedicatedTokenizationBodyTypedDict",
    "DedicatedTokenizationRequest",
    "DedicatedTokenizationRequestTypedDict",
    "Delta",
    "DeltaTypedDict",
    "DetokenizationResult",
    "DetokenizationResultTypedDict",
    "Error",
    "ErrorTypedDict",
    "FileBuiltInTool",
    "FileBuiltInToolType",
    "FileBuiltInToolTypedDict",
    "Files",
    "FilesTypedDict",
    "FinishReason",
    "Function",
    "FunctionTool",
    "FunctionToolType",
    "FunctionToolTypedDict",
    "FunctionTypedDict",
    "JSONSchema",
    "JSONSchemaTypedDict",
    "KnowledgeRetrieveResult",
    "KnowledgeRetrieveResultTypedDict",
    "LogitBias",
    "LogitBiasTypedDict",
    "Message",
    "MessageTypedDict",
    "Name",
    "Object",
    "OtherBuiltInTool",
    "OtherBuiltInToolType",
    "OtherBuiltInToolTypedDict",
    "Parameters",
    "ParametersTypedDict",
    "Prompt",
    "PromptTypedDict",
    "ResponseFormat",
    "ResponseFormatJSONObject",
    "ResponseFormatJSONObjectType",
    "ResponseFormatJSONObjectTypedDict",
    "ResponseFormatJSONSchema",
    "ResponseFormatJSONSchemaTypedDict",
    "ResponseFormatRegex",
    "ResponseFormatRegexType",
    "ResponseFormatRegexTypedDict",
    "ResponseFormatText",
    "ResponseFormatTextType",
    "ResponseFormatTextTypedDict",
    "ResponseFormatTypedDict",
    "Results",
    "ResultsTypedDict",
    "Role",
    "SDKError",
    "Schema",
    "SchemaTypedDict",
    "Security",
    "SecurityTypedDict",
    "ServerlessChatCompleteBody",
    "ServerlessChatCompleteBodyTypedDict",
    "ServerlessChatCompleteRequest",
    "ServerlessChatCompleteRequestTypedDict",
    "ServerlessChatStreamBody",
    "ServerlessChatStreamBodyLogitBias",
    "ServerlessChatStreamBodyLogitBiasTypedDict",
    "ServerlessChatStreamBodyStreamOptions",
    "ServerlessChatStreamBodyStreamOptionsTypedDict",
    "ServerlessChatStreamBodyToolChoice",
    "ServerlessChatStreamBodyToolChoiceFunction",
    "ServerlessChatStreamBodyToolChoiceFunctionTypedDict",
    "ServerlessChatStreamBodyToolChoiceObject",
    "ServerlessChatStreamBodyToolChoiceObjectTypedDict",
    "ServerlessChatStreamBodyToolChoiceType",
    "ServerlessChatStreamBodyToolChoiceTypedDict",
    "ServerlessChatStreamBodyTypedDict",
    "ServerlessChatStreamRequest",
    "ServerlessChatStreamRequestTypedDict",
    "ServerlessCompletionsCompleteBody",
    "ServerlessCompletionsCompleteBodyTypedDict",
    "ServerlessCompletionsCompleteRequest",
    "ServerlessCompletionsCompleteRequestTypedDict",
    "ServerlessCompletionsStreamBody",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithPrompt",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithPromptTypedDict",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithTokens",
    "ServerlessCompletionsStreamBodyCompletionsBodyWithTokensTypedDict",
    "ServerlessCompletionsStreamBodyTypedDict",
    "ServerlessCompletionsStreamRequest",
    "ServerlessCompletionsStreamRequestTypedDict",
    "ServerlessDetokenizationBody",
    "ServerlessDetokenizationBodyTypedDict",
    "ServerlessDetokenizationRequest",
    "ServerlessDetokenizationRequestTypedDict",
    "ServerlessKnowledgeRetrieveBody",
    "ServerlessKnowledgeRetrieveBodyTypedDict",
    "ServerlessKnowledgeRetrieveRequest",
    "ServerlessKnowledgeRetrieveRequestTypedDict",
    "ServerlessTokenizationBody",
    "ServerlessTokenizationBodyTypedDict",
    "ServerlessTokenizationRequest",
    "ServerlessTokenizationRequestTypedDict",
    "ServerlessToolAssistedChatCompleteBody",
    "ServerlessToolAssistedChatCompleteBodyLogitBias",
    "ServerlessToolAssistedChatCompleteBodyLogitBiasTypedDict",
    "ServerlessToolAssistedChatCompleteBodyStreamOptions",
    "ServerlessToolAssistedChatCompleteBodyStreamOptionsTypedDict",
    "ServerlessToolAssistedChatCompleteBodyToolChoice",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceFunction",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceFunctionTypedDict",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceObject",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceObjectTypedDict",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceType",
    "ServerlessToolAssistedChatCompleteBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatCompleteBodyTypedDict",
    "ServerlessToolAssistedChatCompleteRequest",
    "ServerlessToolAssistedChatCompleteRequestTypedDict",
    "ServerlessToolAssistedChatStreamBody",
    "ServerlessToolAssistedChatStreamBodyLogitBias",
    "ServerlessToolAssistedChatStreamBodyLogitBiasTypedDict",
    "ServerlessToolAssistedChatStreamBodyStreamOptions",
    "ServerlessToolAssistedChatStreamBodyStreamOptionsTypedDict",
    "ServerlessToolAssistedChatStreamBodyToolChoice",
    "ServerlessToolAssistedChatStreamBodyToolChoiceFunction",
    "ServerlessToolAssistedChatStreamBodyToolChoiceFunctionTypedDict",
    "ServerlessToolAssistedChatStreamBodyToolChoiceObject",
    "ServerlessToolAssistedChatStreamBodyToolChoiceObjectTypedDict",
    "ServerlessToolAssistedChatStreamBodyToolChoiceType",
    "ServerlessToolAssistedChatStreamBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatStreamBodyTypedDict",
    "ServerlessToolAssistedChatStreamRequest",
    "ServerlessToolAssistedChatStreamRequestTypedDict",
    "Status",
    "StreamOptions",
    "StreamOptionsTypedDict",
    "StreamedChatChoice",
    "StreamedChatChoiceFinishReason",
    "StreamedChatChoiceFunction",
    "StreamedChatChoiceFunctionTypedDict",
    "StreamedChatChoiceToolCalls",
    "StreamedChatChoiceToolCallsTypedDict",
    "StreamedChatChoiceType",
    "StreamedChatChoiceTypedDict",
    "StreamedChatResult",
    "StreamedChatResultObject",
    "StreamedChatResultTypedDict",
    "StreamedChatResultUsage",
    "StreamedChatResultUsageTypedDict",
    "StreamedCompletionsChoice",
    "StreamedCompletionsChoiceFinishReason",
    "StreamedCompletionsChoiceTypedDict",
    "StreamedCompletionsResult",
    "StreamedCompletionsResultData",
    "StreamedCompletionsResultDataTypedDict",
    "StreamedCompletionsResultObject",
    "StreamedCompletionsResultTypedDict",
    "StreamedCompletionsResultUsage",
    "StreamedCompletionsResultUsageTypedDict",
    "StreamedToolAssistedChatResult",
    "StreamedToolAssistedChatResultTypedDict",
    "StreamedToolAssistedChatToken",
    "StreamedToolAssistedChatTokenData",
    "StreamedToolAssistedChatTokenDataTypedDict",
    "StreamedToolAssistedChatTokenObject",
    "StreamedToolAssistedChatTokenTypedDict",
    "StreamedToolAssistedChatTokenUsage",
    "StreamedToolAssistedChatTokenUsageTypedDict",
    "StreamedToolAssistedChatToolStatus",
    "StreamedToolAssistedChatToolStatusData",
    "StreamedToolAssistedChatToolStatusDataTypedDict",
    "StreamedToolAssistedChatToolStatusParameters",
    "StreamedToolAssistedChatToolStatusParametersTypedDict",
    "StreamedToolAssistedChatToolStatusTypedDict",
    "SystemMessage",
    "SystemMessageTypedDict",
    "TokenSequence",
    "TokenSequenceTypedDict",
    "TokenizationResult",
    "TokenizationResultTypedDict",
    "Tool",
    "ToolAssistedChatTool",
    "ToolAssistedChatToolTypedDict",
    "ToolCalls",
    "ToolCallsTypedDict",
    "ToolChoice",
    "ToolChoiceFunction",
    "ToolChoiceFunctionTypedDict",
    "ToolChoiceObject",
    "ToolChoiceObjectTypedDict",
    "ToolChoiceType",
    "ToolChoiceTypedDict",
    "ToolMessage",
    "ToolMessageRole",
    "ToolMessageTypedDict",
    "ToolType",
    "ToolTypedDict",
    "TopLogprobs",
    "TopLogprobsTypedDict",
    "Type",
    "Usage",
    "UsageTypedDict",
    "UserMessage",
    "UserMessageRole",
    "UserMessageTypedDict",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


Role = Literal["system"]
r"""The role of the messages author."""


class SystemMessageTypedDict(TypedDict):
    role: Role
    r"""The role of the messages author."""
    content: str
    r"""The content of system message."""
    name: NotRequired[str]
    r"""The name for the participant to distinguish between participants with the same role."""


class SystemMessage(BaseModel):
    role: Role
    r"""The role of the messages author."""

    content: str
    r"""The content of system message."""

    name: Optional[str] = None
    r"""The name for the participant to distinguish between participants with the same role."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from friendli import models, utils
from friendli._hooks import HookContext
from friendli.types import OptionalNullable, UNSET
from friendli.utils import eventstreaming, get_security_from_env
from typing import List, Mapping, Optional, Union


class FriendliChat(BaseSDK):
    def complete(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[
            Union[
                models.DedicatedChatCompleteBodyLogitBias,
                models.DedicatedChatCompleteBodyLogitBiasTypedDict,
            ]
        ] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = False,
        stream_options: OptionalNullable[
            Union[
                models.DedicatedChatCompleteBodyStreamOptions,
                models.DedicatedChatCompleteBodyStreamOptionsTypedDict,
            ]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.DedicatedChatCompleteBodyToolChoice,
                models.DedicatedChatCompleteBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_logprobs: OptionalNullable[int] = UNSET,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatResult:
        r"""Chat completions

        Given a list of messages forming a conversation, the model generates a response.

        :param model: ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format.
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param logit_bias: Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model.
        :param logprobs: Whether to return log probabilities of the output tokens or not.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` or `response_format` is specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format:
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param stream_options: Options related to stream. It can only be used when `stream: true`.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. Currently, only functions are supported as a tool. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for.  **When `tools` is specified, `min_tokens` and `response_format` fields are unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_logprobs: The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedChatCompleteRequest(
            x_friendli_team=x_friendli_team,
            dedicated_chat_complete_body=models.DedicatedChatCompleteBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                logit_bias=utils.get_pydantic_model(
                    logit_bias,
                    OptionalNullable[models.DedicatedChatCompleteBodyLogitBias],
                ),
                logprobs=logprobs,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, Optional[models.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options,
                    OptionalNullable[models.DedicatedChatCompleteBodyStreamOptions],
                ),
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[models.DedicatedChatCompleteBodyToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.Tool]]
                ),
                top_k=top_k,
                top_logprobs=top_logprobs,
                top_p=top_p,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/dedicated/v1/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_chat_complete_body,
                False,
                False,
                "json",
                models.DedicatedChatCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="dedicatedChatComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ChatResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def complete_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[
            Union[
                models.DedicatedChatCompleteBodyLogitBias,
                models.DedicatedChatCompleteBodyLogitBiasTypedDict,
            ]
        ] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = False,
        stream_options: OptionalNullable[
            Union[
                models.DedicatedChatCompleteBodyStreamOptions,
                models.DedicatedChatCompleteBodyStreamOptionsTypedDict,
            ]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.DedicatedChatCompleteBodyToolChoice,
                models.DedicatedChatCompleteBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_logprobs: OptionalNullable[int] = UNSET,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatResult:
        r"""Chat completions

        Given a list of messages forming a conversation, the model generates a response.

        :param model: ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format.
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param logit_bias: Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model.
        :param logprobs: Whether to return log probabilities of the output tokens or not.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` or `response_format` is specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format:
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param stream_options: Options related to stream. It can only be used when `stream: true`.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. Currently, only functions are supported as a tool. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for.  **When `tools` is specified, `min_tokens` and `response_format` fields are unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_logprobs: The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedChatCompleteRequest(
            x_friendli_team=x_friendli_team,
            dedicated_chat_complete_body=models.DedicatedChatCompleteBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                logit_bias=utils.get_pydantic_model(
                    logit_bias,
                    OptionalNullable[models.DedicatedChatCompleteBodyLogitBias],
                ),
                logprobs=logprobs,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, Optional[models.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options,
                    OptionalNullable[models.DedicatedChatCompleteBodyStreamOptions],
                ),
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[models.DedicatedChatCompleteBodyToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.Tool]]
                ),
                top_k=top_k,
                top_logprobs=top_logprobs,
                top_p=top_p,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/dedicated/v1/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_chat_complete_body,
                False,
                False,
                "json",
                models.DedicatedChatCompleteBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="dedicatedChatComplete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.ChatResult)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def stream(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[
            Union[
                models.DedicatedChatStreamBodyLogitBias,
                models.DedicatedChatStreamBodyLogitBiasTypedDict,
            ]
        ] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = True,
        stream_options: OptionalNullable[
            Union[
                models.DedicatedChatStreamBodyStreamOptions,
                models.DedicatedChatStreamBodyStreamOptionsTypedDict,
            ]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.DedicatedChatStreamBodyToolChoice,
                models.DedicatedChatStreamBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_logprobs: OptionalNullable[int] = UNSET,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.StreamedChatResult]:
        r"""Stream chat completions

        Given a list of messages forming a conversation, the model generates a response.

        :param model: ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format.
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param logit_bias: Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model.
        :param logprobs: Whether to return log probabilities of the output tokens or not.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` or `response_format` is specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format:
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param stream_options: Options related to stream. It can only be used when `stream: true`.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. Currently, only functions are supported as a tool. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for.  **When `tools` is specified, `min_tokens` and `response_format` fields are unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_logprobs: The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedChatStreamRequest(
            x_friendli_team=x_friendli_team,
            dedicated_chat_stream_body=models.DedicatedChatStreamBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                logit_bias=utils.get_pydantic_model(
                    logit_bias,
                    OptionalNullable[models.DedicatedChatStreamBodyLogitBias],
                ),
                logprobs=logprobs,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, Optional[models.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options,
                    OptionalNullable[models.DedicatedChatStreamBodyStreamOptions],
                ),
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[models.DedicatedChatStreamBodyToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.Tool]]
                ),
                top_k=top_k,
                top_logprobs=top_logprobs,
                top_p=top_p,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/dedicated/v1/chat/completions#stream",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_chat_stream_body,
                False,
                False,
                "json",
                models.DedicatedChatStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="dedicatedChatStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.StreamedChatResult),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def stream_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        x_friendli_team: Optional[str] = None,
        eos_token: OptionalNullable[List[int]] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[
            Union[
                models.DedicatedChatStreamBodyLogitBias,
                models.DedicatedChatStreamBodyLogitBiasTypedDict,
            ]
        ] = UNSET,
        logprobs: OptionalNullable[bool] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        min_tokens: OptionalNullable[int] = 0,
        n: OptionalNullable[int] = 1,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        repetition_penalty: OptionalNullable[float] = UNSET,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: OptionalNullable[List[int]] = UNSET,
        stop: OptionalNullable[List[str]] = UNSET,
        stream: OptionalNullable[bool] = True,
        stream_options: OptionalNullable[
            Union[
                models.DedicatedChatStreamBodyStreamOptions,
                models.DedicatedChatStreamBodyStreamOptionsTypedDict,
            ]
        ] = UNSET,
        temperature: OptionalNullable[float] = 1,
        timeout_microseconds: OptionalNullable[int] = UNSET,
        tool_choice: Optional[
            Union[
                models.DedicatedChatStreamBodyToolChoice,
                models.DedicatedChatStreamBodyToolChoiceTypedDict,
            ]
        ] = None,
        tools: OptionalNullable[
            Union[List[models.Tool], List[models.ToolTypedDict]]
        ] = UNSET,
        top_k: OptionalNullable[int] = 0,
        top_logprobs: OptionalNullable[int] = UNSET,
        top_p: OptionalNullable[float] = 1,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.StreamedChatResult]:
        r"""Stream chat completions

        Given a list of messages forming a conversation, the model generates a response.

        :param model: ID of target endpoint. If you want to send request to specific adapter, using \"ENDPOINT_ID:ADAPTER_ROUTE\" format.
        :param messages: A list of messages comprising the conversation so far.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param eos_token: A list of endpoint sentence tokens.
        :param frequency_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled, taking into account their frequency in the preceding text. This penalization diminishes the model's tendency to reproduce identical lines verbatim.
        :param logit_bias: Accepts a JSON object that maps tokens to an associated bias value. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model.
        :param logprobs: Whether to return log probabilities of the output tokens or not.
        :param max_tokens: The maximum number of tokens to generate. For decoder-only models like GPT, the length of your input tokens plus `max_tokens` should not exceed the model's maximum length (e.g., 2048 for OpenAI GPT-3). For encoder-decoder models like T5 or BlenderBot, `max_tokens` should not exceed the model's maximum output length. This is similar to Hugging Face's [`max_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.max_new_tokens) argument.
        :param min_tokens: The minimum number of tokens to generate. Default value is 0. This is similar to Hugging Face's [`min_new_tokens`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.min_new_tokens) argument.  **This field is unsupported when `tools` or `response_format` is specified.**
        :param n: The number of independently generated results for the prompt. Not supported when using beam search. Defaults to 1. This is similar to Hugging Face's [`num_return_sequences`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.num_return_sequences) argument.
        :param parallel_tool_calls: Whether to enable parallel function calling.
        :param presence_penalty: Number between -2.0 and 2.0. Positive values penalizes tokens that have been sampled at least once in the existing text.
        :param repetition_penalty: Penalizes tokens that have already appeared in the generated result (plus the input tokens for decoder-only models). Should be greater than or equal to 1.0 (1.0 means no penalty). See [keskar et al., 2019](https://arxiv.org/abs/1909.05858) for more details. This is similar to Hugging Face's [`repetition_penalty`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.repetition_penalty) argument.
        :param response_format:
        :param seed: Seed to control random procedure. If nothing is given, random seed is used for sampling, and return the seed along with the generated result. When using the `n` argument, you can pass a list of seed values to control all of the independent generations.
        :param stop: When one of the stop phrases appears in the generation result, the API will stop generation. The stop phrases are excluded from the result. Defaults to empty list.
        :param stream: Whether to stream generation result. When set true, each token will be sent as [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#event_stream_format) once generated.
        :param stream_options: Options related to stream. It can only be used when `stream: true`.
        :param temperature: Sampling temperature. Smaller temperature makes the generation result closer to greedy, argmax (i.e., `top_k = 1`) sampling. Defaults to 1.0. This is similar to Hugging Face's [`temperature`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.generationconfig.temperature) argument.
        :param timeout_microseconds: Request timeout. Gives the `HTTP 429 Too Many Requests` response status code. Default behavior is no timeout.
        :param tool_choice: Determines the tool calling behavior of the model. When set to `none`, the model will bypass tool execution and generate a response directly. In `auto` mode (the default), the model dynamically decides whether to call a tool or respond with a message. Alternatively, setting `required` ensures that the model invokes at least one tool before responding to the user. You can also specify a particular tool by `{\"type\": \"function\", \"function\": {\"name\": \"my_function\"}}`.
        :param tools: A list of tools the model may call. Currently, only functions are supported as a tool. A maximum of 128 functions is supported. Use this to provide a list of functions the model may generate JSON inputs for.  **When `tools` is specified, `min_tokens` and `response_format` fields are unsupported.**
        :param top_k: The number of highest probability tokens to keep for sampling. Numbers between 0 and the vocab size of the model (both inclusive) are allowed. The default value is 0, which means that the API does not apply top-k filtering. This is similar to Hugging Face's [`top_k`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_k) argument.
        :param top_logprobs: The number of most likely tokens to return at each token position, each with an associated log probability. `logprobs` must be set to true if this parameter is used.
        :param top_p: Tokens comprising the top `top_p` probability mass are kept for sampling. Numbers between 0.0 (exclusive) and 1.0 (inclusive) are allowed. Defaults to 1.0. This is similar to Hugging Face's [`top_p`](https://huggingface.co/docs/transformers/v4.26.0/en/main_classes/text_generation#transformers.GenerationConfig.top_p) argument.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.DedicatedChatStreamRequest(
            x_friendli_team=x_friendli_team,
            dedicated_chat_stream_body=models.DedicatedChatStreamBody(
                model=model,
                messages=utils.get_pydantic_model(messages, List[models.Message]),
                eos_token=eos_token,
                frequency_penalty=frequency_penalty,
                logit_bias=utils.get_pydantic_model(
                    logit_bias,
                    OptionalNullable[models.DedicatedChatStreamBodyLogitBias],
                ),
                logprobs=logprobs,
                max_tokens=max_tokens,
                min_tokens=min_tokens,
                n=n,
                parallel_tool_calls=parallel_tool_calls,
                presence_penalty=presence_penalty,
                repetition_penalty=repetition_penalty,
                response_format=utils.get_pydantic_model(
                    response_format, Optional[models.ResponseFormat]
                ),
                seed=seed,
                stop=stop,
                stream=stream,
                stream_options=utils.get_pydantic_model(
                    stream_options,
                    OptionalNullable[models.DedicatedChatStreamBodyStreamOptions],
                ),
                temperature=temperature,
                timeout_microseconds=timeout_microseconds,
                tool_choice=utils.get_pydantic_model(
                    tool_choice, Optional[models.DedicatedChatStreamBodyToolChoice]
                ),
                tools=utils.get_pydantic_model(
                    tools, OptionalNullable[List[models.Tool]]
                ),
                top_k=top_k,
                top_logprobs=top_logprobs,
                top_p=top_p,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/dedicated/v1/chat/completions#stream",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_chat_stream_body,
                False,
                False,
                "json",
                models.DedicatedChatStreamBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="dedicatedChatStream",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(raw, models.StreamedChatResult),
                sentinel="[DONE]",
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from friendli.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseFormatTextType = Literal["text"]
r"""The type of the response format: `text`"""


class ResponseFormatTextTypedDict(TypedDict):
    type: ResponseFormatTextType
    r"""The type of the response format: `text`"""


class ResponseFormatText(BaseModel):
    type: ResponseFormatTextType
    r"""The type of the response format: `text`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatlogprobs import ChatLogprobs, ChatLogprobsTypedDict
from friendli.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatChoiceType = Literal["function"]
r"""The type of the tool."""


class ChatChoiceFunctionTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the function to call."""
    arguments: NotRequired[str]
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class ChatChoiceFunction(BaseModel):
    name: Optional[str] = None
    r"""The name of the function to call."""

    arguments: Optional[str] = None
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """


class ChatChoiceToolCallsTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The ID of the tool call."""
    type: NotRequired[ChatChoiceType]
    r"""The type of the tool."""
    function: NotRequired[ChatChoiceFunctionTypedDict]


class ChatChoiceToolCalls(BaseModel):
    id: Optional[str] = None
    r"""The ID of the tool call."""

    type: Optional[ChatChoiceType] = None
    r"""The type of the tool."""

    function: Optional[ChatChoiceFunction] = None


class ChatChoiceMessageTypedDict(TypedDict):
    role: NotRequired[str]
    r"""Role of the generated message author, in this case `assistant`."""
    content: NotRequired[Nullable[str]]
    r"""The contents of the assistant message."""
    tool_calls: NotRequired[Nullable[List[ChatChoiceToolCallsTypedDict]]]


class ChatChoiceMessage(BaseModel):
    role: Optional[str] = None
    r"""Role of the generated message author, in this case `assistant`."""

    content: OptionalNullable[str] = UNSET
    r"""The contents of the assistant message."""

    tool_calls: OptionalNullable[List[ChatChoiceToolCalls]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["role", "content", "tool_calls"]
        nullable_fields = ["content", "tool_calls"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


FinishReason = Literal["stop", "length", "tool_calls"]
r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
`length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
`tool_calls` means the API has generated tool calls.

"""


class ChatChoiceTypedDict(TypedDict):
    index: NotRequired[int]
    r"""The index of the choice in the list of generated choices."""
    message: NotRequired[ChatChoiceMessageTypedDict]
    finish_reason: NotRequired[FinishReason]
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """
    logprobs: NotRequired[Nullable[ChatLogprobsTypedDict]]
    r"""Log probability information for the choice."""


class ChatChoice(BaseModel):
    index: Optional[int] = None
    r"""The index of the choice in the list of generated choices."""

    message: Optional[ChatChoiceMessage] = None

    finish_reason: Optional[FinishReason] = None
    r"""Termination condition of the generation. `stop` means the API returned the full chat completions generated by the model without running into any limits.
    `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length.
    `tool_calls` means the API has generated tool calls.

    """

    logprobs: OptionalNullable[ChatLogprobs] = UNSET
    r"""Log probability information for the choice."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["index", "message", "finish_reason", "logprobs"]
        nullable_fields = ["logprobs"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionsbodywithprompt import (
    CompletionsBodyWithPrompt,
    CompletionsBodyWithPromptTypedDict,
)
from .completionsbodywithtokens import (
    CompletionsBodyWithTokens,
    CompletionsBodyWithTokensTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ServerlessCompletionsCompleteBodyTypedDict = TypeAliasType(
    "ServerlessCompletionsCompleteBodyTypedDict",
    Union[CompletionsBodyWithPromptTypedDict, CompletionsBodyWithTokensTypedDict],
)


ServerlessCompletionsCompleteBody = TypeAliasType(
    "ServerlessCompletionsCompleteBody",
    Union[CompletionsBodyWithPrompt, CompletionsBodyWithTokens],
)
